<?php
/*
Plugin Name: Locser Settings
Version: 1.0
Author: Anoop M C
Author URI: http://www.swapitsolutions.com
*/

define( 'LOCSER_PLUGIN_BASENAME', plugin_basename( __FILE__ ) );
define( 'LOCSER_PLUGIN_NAME', trim( dirname( LOCSER_PLUGIN_BASENAME ), '/' ) );
define( 'LOCSER_PLUGIN_DIR', WP_PLUGIN_DIR . '/' . LOCSER_PLUGIN_NAME );
define( 'LOCSER_PLUGIN_URL', WP_PLUGIN_URL . '/' . LOCSER_PLUGIN_NAME );

global $wpdb;
define( 'LOCSER_TABLE',  $wpdb->prefix . 'locser_options' );

//Create a table in MySQL database when activate plugin
function locser_setup() {
	global $wpdb;
	$wpdb->query('
		CREATE TABLE IF NOT EXISTS ' . LOCSER_TABLE . ' (
		  `id` int(5) NOT NULL AUTO_INCREMENT,
		  `label` varchar(100) NOT NULL,
		  `name` varchar(80) NOT NULL,
		  `value` varchar(255) NOT NULL,
		  PRIMARY KEY (`id`)
		) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
	' );
}
register_activation_hook( __FILE__, 'locser_setup' );


//Create a Menu Locser Settings in Settings
add_action('admin_menu', 'locser_add_menu');
function locser_add_menu() {

 	global $my_plugin_hook;
 	$my_plugin_hook = add_options_page('Locser Settings', 'Locser Settings', 'manage_options', 'locser_options', 'locser_options_adm');

}

function locser_insert() {
	global $wpdb;
	
	$_POST['label'] = filter_var($_POST['label'], FILTER_SANITIZE_SPECIAL_CHARS);
	$_POST['name'] 	= filter_var($_POST['name'], FILTER_SANITIZE_SPECIAL_CHARS);
	$_POST['value'] = filter_var($_POST['value'], FILTER_UNSAFE_RAW);	
	
	return $wpdb->insert( 
		LOCSER_TABLE, 
		array( 
			'label' => $_POST['label'], 
			'name' => $_POST['name'],
			'value' => $_POST['value']
		)
	);
}

function locser_update() {
	global $wpdb;
	
	$_POST['id'] 	= filter_var($_POST['id'], FILTER_VALIDATE_INT);
	$_POST['label'] = filter_var($_POST['label'], FILTER_SANITIZE_SPECIAL_CHARS);
	$_POST['name'] 	= filter_var($_POST['name'], FILTER_SANITIZE_SPECIAL_CHARS);
	$_POST['value'] = filter_var($_POST['value'], FILTER_UNSAFE_RAW);
	
	return $wpdb->update(
		LOCSER_TABLE, 
		array( 
			'label' => $_POST['label'], 
			'name' 	=> $_POST['name'],
			'value' => $_POST['value']
		),
		array ('id' => $_POST['id'])
	);

}

function locser_delete( $id ) {
	global $wpdb;
	
	return $wpdb->query($wpdb->prepare('DELETE FROM ' . LOCSER_TABLE . ' WHERE id = \'%d\' ', $id) );
}

function locser_get_options() {
	global $wpdb;
	
	return $wpdb->get_results('SELECT * FROM ' . LOCSER_TABLE . ' ORDER BY label ASC');
}

function locser_get_option( $id ) {
	global $wpdb;
	
	return $wpdb->get_row('SELECT * FROM ' . LOCSER_TABLE . ' WHERE id = ' . $id );
}

//Panel Admin
function locser_options_adm() {
	global $wpdb, $my_plugin_hook;

	$id 	= '';
	$label 	= '';
	$name 	= '';
	$value 	= '';
	
	$message = '';
	
	if ( isset($_GET['del']) && $_GET['del'] > 0 ) :
		if ( locser_delete( $_GET['del'] ) ) :
			$message = '<div class="updated"><p><strong>' . __('Settings saved.') . '</strong></p></div>';
		endif;
		
		
	elseif ( isset($_POST['id']) ) :
		
		if ($_POST['id'] == '') :
			locser_insert();
			$message = '<div class="updated"><p><strong>' . __('Settings saved.') . '</strong></p></div>';			
			
		elseif ($_POST['id'] > 0) :		
			locser_update();
			$message = '<div class="updated"><p><strong>' . __('Settings saved.') . '</strong></p></div>';			
			
		endif;
		
		
	elseif ( isset($_GET['id']) && $_GET['id'] > 0 ) :
		
		$option = locser_get_option( $_GET['id'] );
		
		$id 	= $option->id;
		$label 	= $option->label;
		$name 	= $option->name;
		$value 	= $option->value;
		
	endif;
	
	$options = locser_get_options();
?>
	
	<div class="wrap">
		<div id="icon-tools" class="icon32"></div><h2>Locser Settings</h2>
		
		<?php echo $message; ?>
		<br />
		<?php if ( count($options) > 0 ) : ?>
			<div class="wpbody-content">
				<table class="widefat" cellspacing="0">
					<thead>
						<tr>
							<th>Label</th>
							<th>Name</th>
							<th>Value</th>
							<th> </th>
						</tr>
					</thead>
					<tbody>
						<?php foreach ($options as $option ) : ?>
						<tr>
							<td><a title="Edit <?php echo $option->label; ?>" href="<?php echo preg_replace('/\\&.*/', '', $_SERVER['REQUEST_URI']); ?>&id=<?php echo $option->id; ?>"><?php echo $option->label; ?></a></td>
							<td><a title="Edit <?php echo $option->label; ?>" href="<?php echo preg_replace('/\\&.*/', '', $_SERVER['REQUEST_URI']); ?>&id=<?php echo $option->id; ?>"><?php echo $option->name; ?></a></td>
							<td><a title="Edit <?php echo $option->label; ?>" href="<?php echo preg_replace('/\\&.*/', '', $_SERVER['REQUEST_URI']); ?>&id=<?php echo $option->id; ?>"><?php echo $option->value; ?></a></td>
							<td><span class="trash"><a onclick="return confirm('Are you sure want to delete item?')" class="submitdelete" title="Delete <?php echo $option->label; ?>" href="<?php echo preg_replace('/\\&.*/', '', $_SERVER['REQUEST_URI']); ?>&del=<?php echo $option->id; ?>">Delete</a></span></td>
						</tr>
						<?php endforeach; ?>
					</tbody>
				</table>
			</div>
		<br />
		<?php endif; ?>
		
		<form method="post" action="<?php echo preg_replace('/\\&.*/', '', $_SERVER['REQUEST_URI']); ?>">
			<input type="hidden" name="id" value="<?php echo $id; ?>" />
			<h3>Add new Settings</h3>
			<table class="form-table">				
				<tbody>
					<tr valign="top">
						<th scope="row">
							<label for="label">Label:</label>
						</td>
						<td>
							<input name="label" type="text" id="label" value="<?php echo $label; ?>" class="regular-text">
						</td>
					</tr>
					<tr>
						<th scope="row">
							<label for="name">*Name:</label>
						</td>
						<td>
							<input name="name" type="text" id="name" value="<?php echo $name; ?>" class="regular-text">
						</td>						
					</tr>
					<tr>
						<th scope="row">
							<label for="value">Value:</label>
						</td>
						<td>
							<textarea name="value" rows="7" cols="40" type="text" id="value" class="regular-text code"><?php echo $value; ?></textarea>
						</td>						
					</tr>
				</tbody>
			</table>
			<p class="submit"><input type="submit" name="submit" id="submit" class="button-primary" value="<?php _e('Save Changes'); ?>"></p>
		</form>
		
	</div>
<?php
}



//get your single option
function get_custom( $name ) {
	global $wpdb;
	if ( '' != $name ) :
		return $wpdb->get_var( $wpdb->prepare( 'SELECT value FROM ' . LOCSER_TABLE . ' WHERE name = \'%s\' LIMIT 1', $name ) );   
	else :
		return false;
	endif;
}

//get your array options
function get_customs( $name ) {
	global $wpdb;
	if ( '' != $name ) :
		$list = $wpdb->get_results( $wpdb->prepare( 'SELECT value FROM ' . LOCSER_TABLE . ' WHERE name = \'%s\' ', $name ) , ARRAY_A);
		$array = array();
		foreach ( $list as $key => $name ) :
			$array[] = $name['value'];
		endforeach;
		return $array;
	else :
		return false;
	endif;
}


//Tutorial em Help Button
function locser_plugin_help($contextual_help, $screen_id, $screen) {

	global $my_plugin_hook;
	if ($screen_id == $my_plugin_hook) {

		$contextual_help = 'Used <em>get_custom(\'name\')</em> or <em>get_customs(\'name\')</em> in your theme.';
	}
	return $contextual_help;
}

add_filter('contextual_help', 'locser_plugin_help', 10, 3);